/**
 * 1、产出语言包json供mis导入
 * 2、从mis拉取全量语言包配置
 * 3、读取模块下的lang.json，如果跟mis拉取的不同以mis为准
 * 4、如果模块下没有lang.json或lang.json里没有文字的语言配置，则用mis的更新模块的lang.json
 * 5、读取mis的配置，键值优先匹配全部，如果没有则以文字的md5为key查找
 * 6、如果都没有找到，则自动翻译中文简体为繁体
 */
var fs = require('fs');
var convert = require('chinese_convert');
var _ = require('lodash');
var sr = require('sync-request');

var projectName = fis.project.getProjectPath().split('/').pop();
var reportJson = {};
var misDataHolder;
var localDataHolder = {
    'zh_tw': require('./fe-zh-TW_lang.json')
}

/**
 * 根据语言获取字体
 *
 * @param  {Array}  langPkg 根据语言获取字体
 * @param  {string} zh_cn   中文内容
 * @param  {string} lang    要转的语言
 * @param  {string} pkgPath 语言包文件的路径
 * @return {string}         查询出来的语言文字
 */
var getTextByLang = function (langPkg, zh_cn, lang, pkgPath) {
    var text;

    langPkg.forEach(function (item) {
        if (zh_cn == item.zh_cn) {
            text = item[lang];
        }
    });

    if (text) {
        return text;
    }
    console.log('can not find "' + zh_cn + '" in ' + pkgPath);

    return '';
};


/**
 * 处理js文件及css文件中的多语言逻辑
 *
 * @param  {string} content 文件内容
 * @param  {string} lang    使用语言
 * @return {string}         处理后的内容
 */
var handleJsAndCss = function (content, lang) {

    return content.replace(
        /\/\*\*\s*start\s*:\s*(\S*?)\s*\*\*\/([\s\S]*?)\/\*\*\s*end\s*:\s*(\1)\s*\*\*\//g,
        function (match, start, section, end) {
            if (start == end && start == lang) {
                return section;
            }
            return '';
        }
    );
};

/**
 * 处理html文件中的多语言逻辑
 *
 * @param  {string} content 文件内容
 * @param  {string} lang    使用语言
 * @return {string}         处理后的内容
 */
var handleHtml = function (content, lang) {
    return content.replace(
        /<!--\s*(\S*?)\s*-->([\s\S]*?)<!--\s*\/\s*(\1)\s*-->/g,
        function (match, start, section, end) {
            if (start == end && start == lang) {
                return section;
            }
            return '';
        }
    );
};

/**
 * 处理deploy
 *
 * @param  {Object} obj    值
 * @param  {Object} parent 原始对象
 * @param  {string} key    key
 * @param  {Object} opt    配置
 */
var handleVariable = function (obj, parent, key, opt) {

    if (Object.prototype.toString.call(obj) == '[object Object]') {
        for (var key in obj) {
            handleVariable(obj[key], obj, key, opt);
        }
    }
    else if (Object.prototype.toString.call(obj) == '[object Array]') {
        obj.forEach(function (item, index) {
            handleVariable(obj[index], obj, index, opt);
        });
    }
    else {
        parent[key] = (parent[key] + '').replace(/\$\{\s*(.*?)\s*\}/g, function (match, variable) {
            return opt[variable] || '';
        });
    }
};

function getWordKey (file, word) {
    var namespace = projectName;
    var subPath = file.subpathNoExt;
    if (subPath[0] == '/') {
        subPath = subPath.substr(1);
    }
    if (subPath.match(/^component_modules\//)) {
        namespace = 'component_modules';
        subPath = subPath.replace(/^component_modules\//, '');
    }
    return [namespace, subPath.replace(/\//g, '_'), fis.util.md5(word, 16)].join('_');
}

function handleChineseLine (content, file, lang, langFilePath, misData) {
    misData = misData || {};
    var filePath = file.realpath;
    if (content.indexOf('skip fis-prepackager-beatles-i18n') != -1) {
        return content;
    }
    if (filePath.match(/\/node_modules\/chinese_convert\//)
        || filePath.match(/\/lang\./)
    ) {
        return content;
    }

    var ary = content.toString().split('\n');
    var res = [];
    var localLang = [];
    var needSave = false;
    var notInCode = false;
    if (fs.existsSync(langFilePath)) {
        localLang = require(langFilePath);
    }
    ary = ary.map(function(value, index) {
        // 注释中的中文跳过
        if (value.trim().match(/^\/\*/)) {
            notInCode = true;
            if (value.trim().match(/\*\/$/)) {
                notInCode = false;
            }
            return value;
        }
        if (value.trim().match(/\*\/$/)) {
            notInCode = false;
            return value;
        }
        if (notInCode) {
            return value;
        }
        if(value.trim().match(/^(\*|\/\/|\/\*)/)) {
            return value;
        }
        var reg = /[\u4e00-\u9fa5]+(?:[\u4e00-\u9fa5]|\s|[^a-zA_Z0-9<\/\\>'"#\{\}$&\-\[\]\*])+/g;
        var match = value.match(reg);
        var splicePos = value.indexOf('//');
 
        if (match) {
            if (splicePos != -1 && value.substr(splicePos).indexOf(match[0]) != -1) {
                return value;
            }
            value = value.replace(reg, function (m, pos) {
                var itemKey = getWordKey(file, m);
                var localItem = _.find(localLang, {key: itemKey});
                var v = '';
                if (!localItem) {
                    if (misData[itemKey]) {
                        v = misData[itemKey];
                    } else if (misData[fis.util.md5(m, 16)]) {
                        v = misData[fis.util.md5(m, 16)];
                    }
                    if (v) {
                        var o = {};
                        o['key'] = itemKey;
                        o['zh_cn'] = m;
                        o[lang] = v;
                        localLang.push(o);
                        needSave = true;
                    }
                } else if (localItem && !localItem[lang]) {
                    if (misData[itemKey]) {
                        v = misData[itemKey];
                    } else if (misData[fis.util.md5(m, 16)]) {
                        v = misData[fis.util.md5(m, 16)];
                    }
                    if (v) {
                        localItem[lang] = v;
                        needSave = true;
                    }
                } else {
                    if (misData[itemKey] && (misData[itemKey] != localItem[lang])) {
                       localItem[lang] = v = misData[itemKey];
                       needSave = true;
                    } else {
                       v = localItem[lang];
                    }
                }
                /**
                 * 繁体地名自动翻译不导出;
                 * test目录不处理;
                 * 本身是繁体不处理
                 */ 
                if (lang == 'zh_tw' && filePath.match(/\/(node_modules\/mockjs|didi-component-cityList)\//)
                    || filePath.match(/\/(test)\//)
                    || convert.cn2tw(m) == m
                ) {
                    
                } else {
                    reportJson[getWordKey(file, m)] = m;
                }

                if (v) {
                    return v;
                } else {
                    if (lang == 'zh_tw') {
                        return convert.cn2tw(m);
                    } else {
                        return m;
                    }
                }
            });
            res.push(['文件：' + filePath, '行数：' + (index - 1), '文字：' + match.join(' '), '\n'].join('\n'));
            
            return value;
        }
        return value;
    });
    if (needSave 
        && file.subpathNoExt.match(/^\/component_modules\//)
    ) {
        fis.util.write(langFilePath, JSON.stringify(localLang.sort()));
    }
    fis.util.write('./lang.log', res.join('\n'), null, true);
    fis.util.write('./langmap.json', JSON.stringify(reportJson))
    return ary.join('\n');
}

// 清空旧日志
fis.util.write('./lang.log', '');

var misData = {};
var langFileName = {
    zh_tw: 'zh-TW',
    zh_cn: 'zh-CN'
};
module.exports = function (content, file, settings) {

    // handleVariable(fis.config.get('deploy'), null, null, opt);
    var lang =  settings.lang;
    if (!file.isHtmlLike && !file.isJsLike && !file.isCssLike) {
        return content;
    }
    // 获取mis语言包
    if (lang != 'zh_cn' && _.isEmpty(misData)) {
        try {
            var langUrl = 'https://sfwb.didistatic.com/static/wb/fe-' + langFileName[lang] + '_lang.json?t=' + Date.now();
            var misDataHolder = sr('GET', langUrl);
            misData = JSON.parse(misDataHolder.getBody());
            if (misData.status && misData.status == 'failure') {
               console.log('ERROR: 获取mis翻译包失败，使用内置包代替！');
               misData = localDataHolder[lang] || {};
            } else {
		console.log('获取mis翻译包成功');
	    }
        } catch (err) {
	    console.log('ERROR: 获取mis翻译包失败，使用内置包代替！');
            misData = localDataHolder[lang] || {};
        }
    }

    var langFilePath = file.origin.replace(
        /(.*)(template|page|component_modules|components)(\/[^/]*\/)(.*)/g,
        function (match, project, dir, feature) {
            return project + dir + feature;
        }
    ) + 'lang.json';

    // 类html
    if (/\.(html|tpl|tmpl)$/g.test(file.origin)) {
        content = handleHtml(content, lang)
    }
    // js和css
    else if (/\.(js|css|less|sass|styl)$/g.test(file.origin)) {
        content = handleJsAndCss(content, lang);
    }
    if (lang != 'zh_cn') {
        // 找到所有的中文并把行号信息输出到报告
        content = handleChineseLine(content, file, lang, langFilePath, misData);
    }
    return content;
};
